
/*  ___   ___  ___  _   _  ___   ___   ____ ___  ____  
 * / _ \ /___)/ _ \| | | |/ _ \ / _ \ / ___) _ \|    \ 
 *| |_| |___ | |_| | |_| | |_| | |_| ( (__| |_| | | | |
 * \___/(___/ \___/ \__  |\___/ \___(_)____)___/|_|_|_|
 *                  (____/ 
 * OSOYOO IoT Tutorial: 
 * Make a home access system
 * For more details see:https://osoyoo.com/?p=53322
 */
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <Servo.h>

// WiFi settings
const char* ssid = "your ssid";
const char* password = "your password";

// UDP config
const int udpPort = 8888;
WiFiUDP udp;
IPAddress allowedIP1(192, 168, 0, 5);  // first permission ip
IPAddress allowedIP2(192, 168, 0, 101);  // sencond permission ip

// Servo
Servo servo;
const int servoPin = D3;  // connect the servo to D3

// LCD I2C address settings
LiquidCrystal_I2C lcd(0x27, 16, 2);

void setup() {
  Serial.begin(115200);

  // Connect to WiFi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi...");
  }
  Serial.println("Connected to WiFi");
  Serial.println("IP address is: ");
  Serial.println(WiFi.localIP());
  Serial.printf("UDP server on port %d\n", udpPort);
  // UDP begin
  udp.begin(udpPort);

  // initial the servo
  servo.attach(servoPin);
  servo.write(0);  // set the servo to 0 degree

  // initial LCD
  lcd.init();
  lcd.backlight();
  lcd.setCursor(3, 0);
  lcd.print("Smart home ");
  lcd.setCursor(2, 1);
  lcd.print("access system");
}

void loop() {
  int packetSize = udp.parsePacket();
  if (packetSize) {
  Serial.printf("Received packet of size %d from %s:%d\n    (to %s:%d, free heap = %d B)\n",
                  packetSize,
                  udp.remoteIP().toString().c_str(), udp.remotePort(),
                  udp.destinationIP().toString().c_str(), udp.localPort(),
                  ESP.getFreeHeap());
    IPAddress remoteIP = udp.remoteIP();
    if (remoteIP == allowedIP1) {
      handleAllowedIP1();
    } else if (remoteIP == allowedIP2) {
      handleAllowedIP2();
    } else {
      handleUnauthorizedIP();
    }
  }
}

void handleAllowedIP1() {
  char command = udp.read();
  if (command == 'F') {
    openDoor();
    lcd.clear();
    lcd.print("WELCOME HOME,");
    lcd.setCursor(0, 1);
    lcd.print("ALISA!");
    delay(10000);
    closeDoor();
    lcd.clear();
    lcd.setCursor(3, 0);
    lcd.print("Smart home ");
    lcd.setCursor(2, 1);
    lcd.print("access system");
  }
}

void handleAllowedIP2() {
  char command = udp.read();
  if (command == 'F') {
    openDoor();
    lcd.clear();
    lcd.print("WELCOME HOME,");
    lcd.setCursor(0, 1);
    lcd.print("Elaine!");
    delay(3000);
    closeDoor();
    lcd.clear();
    lcd.setCursor(3, 0);
    lcd.print("Smart home ");
    lcd.setCursor(2, 1);
    lcd.print("access system");
  }
}

void handleUnauthorizedIP() {
  udp.read();  // read data
  lcd.clear();
  lcd.print("NO permission");
  delay(10000);
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("Smart home ");
  lcd.setCursor(2, 1);
  lcd.print("access system");
}

void openDoor() {
  servo.write(90);
  Serial.println("open the door");
  delay(1000);
}

void closeDoor() {
  servo.write(0);
    Serial.println("close the door");

  delay(1000);
}
